@@ -1,17 +1,14 @@
 
     
         
-#   x      a colorSpec object with type 'light'   
+#   x      a colorSpec object with type 'light', and M spectra
 #
-#   returns CCT in Kelvin, or NA 
+#   returns an M-vector, containing CCT in Kelvin, or NA in case of error
 computeCCT.colorSpec   <- function( x, method='robertson', strict=FALSE, c2=1.4388e7 )
     {
     n   = numSpectra( x )
     if( n == 0 )    return( numeric(0) )
 
-    out = rep( NA_real_, n )
-    names(out)  = specnames(x)
-    
     if( type(x) != 'light' )
         {
         log.string( WARN, "The type of of '%s' is '%s', but it must be 'light'.",
@@ -21,45 +18,143 @@ computeCCT.colorSpec   <- function( x, method='robertson', strict=FALSE, c2=1.43
     
     XYZ = product( radiometric(x), colorSpec::xyz1931.1nm, wavelength='auto' )
 
-    for( k in 1:n )
-        {
-        out[k] = CCTfromXYZ( XYZ[k, ], method=method, strict=strict, c2=c2 )
-        }
-    
+    out = CCTfromXYZ( XYZ, method=method, strict=strict, c2=c2 )    
+
     return( out )
     }
         
             
 
     
-#   XYZ     a 3-vector with XYZ
+#   XYZ     an Nx3 matrix, or a vector that can be converted to such a matrix
 #   returns CCT, or NA if outside valid range
-#
-#   requires private data frame dataCCT,  which is lazy-loaded from sysdata.rda;  see savePrivateDatasets()
 
 CCTfromXYZ    <- function( XYZ, method='robertson', strict=FALSE, c2=1.4388e7 )
     {
-    if( any(is.na(XYZ)) )   return( NA_real_ )
+    XYZ = prepareNxM( XYZ, M=3 )
+    if( is.null(XYZ) )   return(NULL)
     
-    if( length(XYZ) != 3 )  return( NA_real_ )
+    n   = nrow(XYZ)
+    out = rep( NA_real_, n )
+    names(out)  = rownames(XYZ)   
     
-    uv  = uv_from_XYZ( XYZ )
-    
-    if( is.na(uv[1]) )  return( uv[1] )
+    if( pmatch( tolower(method), c('mccamy'), nomatch=0 ) != 0 )
+        {
+        # C. S. McCamy "Correlated color temperature as an explicit function of chromaticity coordinates" Color Research & Application Volume 17, Issue 2, pages 142-144, April 1992
+        xyY = xyY_from_XYZ( XYZ )
+        for( i in 1:n )
+            out[i] = CCTfromxy_McCamy( xyY[i,1:2] )
+        }
+    else
+        {
+        for( i in 1:n )
+            {
+            uv      = uv_from_XYZ( XYZ[i, ] )
+            out[i]  = CCTfromuv_single( uv, method=method, strict=strict, c2=c2 ) 
+            }
+        }
             
-    return( CCTfromuv( uv, method=method, strict=strict, c2=c2 ) )
+    return( out )
+    }
+
+
+#   xy  an Nx2 matrix, or a vector that can be converted to such a matrix
+    
+CCTfromxy  <- function( xy, method='robertson', strict=FALSE, c2=1.4388e7 )
+    {
+    xy  = prepareNxM( xy, M=2 )
+    if( is.null(xy) )   return(NULL)
+    
+    n   = nrow(xy)
+    out = rep( NA_real_, n )
+    names(out)  = rownames(xy)    
+    
+    if( pmatch( tolower(method), c('mccamy'), nomatch=0 ) != 0 )
+        {
+        for( i in 1:n )
+            out[i]  = CCTfromxy_McCamy( xy[i, ] )
+        }
+    else
+        {
+        uv  = uv_from_xy( xy, .year=1960 )        
+        for( i in 1:n )
+            out[i]  = CCTfromuv_single( uv[i, ], method=method, strict=strict, c2=c2 )
+        }
+        
+        
+    return(out)
+    }
+
+    
+    
+# C. S. McCamy "Correlated color temperature as an explicit function of chromaticity coordinates" Color Research & Application Volume 17, Issue 2, pages 142-144, April 1992
+
+CCTfromxy_McCamy  <- function( xy )
+    {
+    n = (xy[1] - 0.3320)/(xy[2] - 0.1858)
+    
+    return( -449*n^3 + 3525*n^2 - 6823.3*n + 5520.33 )
     }
     
     
+#   uv  an Nx2 matrix, or a vector that can be converted to such a matrix
+    
 CCTfromuv  <- function( uv, method='robertson', strict=FALSE, c2=1.4388e7 )
     {
-    idx.method  = pmatch( method, c("robertson","lm"), nomatch=0 )
+    uv  = prepareNxM( uv, M=2 )
+    if( is.null(uv) )   return(NULL)
+    
+    n   = nrow(uv)
+    out = rep( NA_real_, n )
+    names(out)  = rownames(uv)
+        
+    if( pmatch( tolower(method), c('mccamy'), nomatch=0 ) != 0 )
+        {
+        for( i in 1:n )
+            {
+            #   McCamy, so convert from uv to xy.  It is unlikely that a user will do this.
+            denom   = uv[i,1] - 4*uv[i,2] + 2
+            
+            if( is.na(denom) ||  denom < 1.e-16 )
+                {
+                # log.string( WARN, "uv=%g,%g is invalid, because denom=%g.",   uv[1], uv[2], denom )
+                }          
+            else
+                {
+                xy      = c( 1.5*uv[i,1], uv[i,2] ) / denom            
+                out[i]  = CCTfromxy_McCamy( xy )
+                }
+            }
+        }
+    else
+        {        
+        for( i in 1:n )
+            out[i]  = CCTfromuv_single( uv[i, ], method=method, strict=strict, c2=c2 )
+        }
+        
+        
+    return(out)
+    }
+
+
+    
+#   uv      a 2-vector with uv 1960
+#   returns CCT, or NA if outside valid range
+#
+#   requires private data frame dataCCT,  which is lazy-loaded from sysdata.rda;  see savePrivateDatasets()
+    
+CCTfromuv_single  <- function( uv, method='robertson', strict=FALSE, c2=1.4388e7 )
+    {
+    if( any( is.na(uv) ) )  return(NA_real_)
+    
+    idx.method  = pmatch( tolower(method), c('robertson','lm'), nomatch=0 )
     if( idx.method == 0 )
         {
         log.string( ERROR, "method='%s' is invalid.", as.character(method) )
         return(  NA_real_ )
         }
 
+
     di = (uv[2] - dataCCT$v) - dataCCT$t * (uv[1] - dataCCT$u)
     #   print( di )
     
@@ -135,15 +230,16 @@ uv_from_XYZ <- function( XYZ )
     {
     denom = XYZ[1]  +  15 * XYZ[2]  +  3 * XYZ[3]
     
-    if( denom < 1.e-16 )
+    if( is.na(denom)  ||  denom < 1.e-16 )
         {
-        log.string( WARN, "XYZ=%g,%g,%g is invalid, because denom=%g.",
-                            XYZ[1], XYZ[2], XYZ[3], denom )
-        return( NA_real_ )
+        # log.string( WARN, "XYZ=%g,%g,%g is invalid, because denom=%g.",  XYZ[1], XYZ[2], XYZ[3], denom )
+        return( c(NA_real_,NA_real_) )
         }    
     
     return( c(4 * XYZ[1], 6 * XYZ[2]) / denom )
     }
+    
+    
 
 #--------       UseMethod() calls           --------------#            
         
